Package: radiant.model
Type: Package
Title: Model Menu for Radiant: Business Analytics using R and Shiny
Version: 1.6.11
Date: 2026-4-14
Authors@R: person("Vincent", "Nijs", , "radiant@rady.ucsd.edu", c("aut", "cre"))
Description: The Radiant Model menu includes interfaces for linear and logistic
    regression, naive Bayes, neural networks, classification and regression trees,
    model evaluation, collaborative filtering, decision analysis, and simulation.
    The application extends the functionality in 'radiant.data'.
Depends: R (>= 4.3.0), radiant.data (>= 1.6.6)
Imports: radiant.basics (>= 1.6.6), shiny (>= 1.8.1), nnet (>= 7.3.12),
        NeuralNetTools (>= 1.5.1), sandwich (>= 2.3.4), car (>= 2.1.3),
        ggplot2 (>= 3.4.2), scales (>= 1.2.1), data.tree (>= 0.7.4),
        stringr (>= 1.1.0), lubridate (>= 1.7.2), tidyr (>= 0.8.2),
        dplyr (>= 1.1.2), tidyselect (>= 1.2.0), rlang (>= 0.4.10),
        magrittr (>= 1.5), DiagrammeR (>= 1.0.9), import (>= 1.1.0),
        psych (>= 1.8.4), e1071 (>= 1.6.8), rpart (>= 4.1.11), ggrepel
        (>= 0.8), broom (>= 0.7.0), patchwork (>= 1.0.0), ranger (>=
        0.11.2), xgboost (>= 1.6.0.1), stringi, yaml
Suggests: testthat (>= 2.0.0), pkgdown (>= 1.1.0)
URL: https://github.com/radiant-rstats/radiant.model/,
        https://radiant-rstats.github.io/radiant.model/,
        https://radiant-rstats.github.io/docs/
BugReports: https://github.com/radiant-rstats/radiant.model/issues/
License: AGPL-3 | file LICENSE
LazyData: true
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-15 06:31:55 UTC; vnijs
Author: Vincent Nijs [aut, cre]
Maintainer: Vincent Nijs <radiant@rady.ucsd.edu>
Repository: CRAN
Date/Publication: 2026-04-15 07:00:08 UTC
Built: R 4.5.2; ; 2026-04-15 09:49:43 UTC; unix
