# Generated by roxygen2: do not edit by hand

S3method(autoplot,partial)
S3method(plot,confusion)
S3method(plot,crs)
S3method(plot,crtree)
S3method(plot,dtree)
S3method(plot,evalbin)
S3method(plot,evalreg)
S3method(plot,gbt)
S3method(plot,logistic)
S3method(plot,mnl)
S3method(plot,mnl.predict)
S3method(plot,model.predict)
S3method(plot,nb)
S3method(plot,nb.predict)
S3method(plot,nn)
S3method(plot,regress)
S3method(plot,repeater)
S3method(plot,rforest)
S3method(plot,rforest.predict)
S3method(plot,simulater)
S3method(plot,uplift)
S3method(predict,crtree)
S3method(predict,gbt)
S3method(predict,logistic)
S3method(predict,mnl)
S3method(predict,nb)
S3method(predict,nn)
S3method(predict,regress)
S3method(predict,rforest)
S3method(print,crtree.predict)
S3method(print,gbt.predict)
S3method(print,logistic.predict)
S3method(print,mnl.predict)
S3method(print,nb.predict)
S3method(print,nn.predict)
S3method(print,regress.predict)
S3method(print,rforest.predict)
S3method(render,DiagrammeR)
S3method(sensitivity,dtree)
S3method(store,crs)
S3method(store,mnl.predict)
S3method(store,model)
S3method(store,model.predict)
S3method(store,nb.predict)
S3method(store,rforest.predict)
S3method(summary,confusion)
S3method(summary,crs)
S3method(summary,crtree)
S3method(summary,dtree)
S3method(summary,evalbin)
S3method(summary,evalreg)
S3method(summary,gbt)
S3method(summary,logistic)
S3method(summary,mnl)
S3method(summary,nb)
S3method(summary,nn)
S3method(summary,regress)
S3method(summary,repeater)
S3method(summary,rforest)
S3method(summary,simulater)
S3method(summary,uplift)
export(.as_int)
export(.as_num)
export(MAE)
export(RMSE)
export(Rsq)
export(ann)
export(auc)
export(confint_robust)
export(confusion)
export(crs)
export(crtree)
export(cv.crtree)
export(cv.gbt)
export(cv.nn)
export(cv.rforest)
export(dtree)
export(dtree_parser)
export(evalbin)
export(evalreg)
export(find_max)
export(find_min)
export(gbt)
export(logistic)
export(minmax)
export(mnl)
export(nb)
export(nn)
export(onehot)
export(pdp_plot)
export(pred_plot)
export(predict_model)
export(print_predict_model)
export(profit)
export(radiant.model)
export(radiant.model_viewer)
export(radiant.model_window)
export(regress)
export(remove_comments)
export(repeater)
export(rforest)
export(rig)
export(scale_df)
export(sdw)
export(sensitivity)
export(sim_cleaner)
export(sim_cor)
export(sim_splitter)
export(sim_summary)
export(simulater)
export(test_specs)
export(uplift)
export(var_check)
export(varimp)
export(varimp_plot)
export(write.coeff)
import(ggplot2)
import(radiant.data)
import(shiny)
importFrom(DiagrammeR,DiagrammeR)
importFrom(DiagrammeR,DiagrammeROutput)
importFrom(DiagrammeR,mermaid)
importFrom(DiagrammeR,renderDiagrammeR)
importFrom(NeuralNetTools,garson)
importFrom(NeuralNetTools,olden)
importFrom(NeuralNetTools,plotnet)
importFrom(broom,augment)
importFrom(car,linearHypothesis)
importFrom(car,vif)
importFrom(data.tree,Clone)
importFrom(data.tree,FormatPercent)
importFrom(data.tree,Get)
importFrom(data.tree,Traverse)
importFrom(data.tree,as.Node)
importFrom(data.tree,isLeaf)
importFrom(data.tree,isNotLeaf)
importFrom(data.tree,isNotRoot)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,arrange_at)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,data_frame)
importFrom(dplyr,desc)
importFrom(dplyr,distinct_at)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,funs)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_)
importFrom(dplyr,group_by_at)
importFrom(dplyr,inner_join)
importFrom(dplyr,last)
importFrom(dplyr,min_rank)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_at)
importFrom(dplyr,mutate_if)
importFrom(dplyr,near)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,sample_n)
importFrom(dplyr,select)
importFrom(dplyr,select_at)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_)
importFrom(dplyr,summarise_all)
importFrom(dplyr,summarise_at)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(e1071,naiveBayes)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggrepel,geom_text_repel)
importFrom(graphics,par)
importFrom(import,from)
importFrom(lubridate,is.Date)
importFrom(lubridate,now)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,"%T>%")
importFrom(magrittr,extract2)
importFrom(magrittr,set_colnames)
importFrom(magrittr,set_names)
importFrom(magrittr,set_rownames)
importFrom(nnet,nnet)
importFrom(nnet,nnet.formula)
importFrom(patchwork,plot_annotation)
importFrom(patchwork,wrap_plots)
importFrom(psych,cohen.kappa)
importFrom(radiant.data,launch)
importFrom(radiant.data,set_attr)
importFrom(radiant.data,visualize)
importFrom(ranger,ranger)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,parse_exprs)
importFrom(rpart,prune.rpart)
importFrom(rpart,rpart)
importFrom(rpart,rpart.control)
importFrom(sandwich,vcovHC)
importFrom(scales,percent)
importFrom(shiny,getDefaultReactiveDomain)
importFrom(shiny,incProgress)
importFrom(shiny,withProgress)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,confint)
importFrom(stats,confint.default)
importFrom(stats,contrasts)
importFrom(stats,cor)
importFrom(stats,deviance)
importFrom(stats,dnorm)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,logLik)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,relevel)
importFrom(stats,residuals)
importFrom(stats,rlnorm)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,step)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(stats,weighted.mean)
importFrom(stats,wilcox.test)
importFrom(stringi,stri_trans_general)
importFrom(stringr,str_match)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(tidyselect,where)
importFrom(utils,as.relistable)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,head)
importFrom(utils,relist)
importFrom(utils,tail)
importFrom(utils,write.table)
importFrom(xgboost,xgb.importance)
importFrom(xgboost,xgboost)
importFrom(yaml,yaml.load)
