> Estimate a Naive Bayes model

To estimate a model select the Response variable and one or more Explanatory variables. Press the `Estimate` button or `CTRL-enter` (`CMD-enter` on mac) to generate results. 

### Report > Rmd

Add code to <a href="https://radiant-rstats.github.io/docs/data/report_rmd.html" target="_blank">_Report > Rmd_</a> to (re)create the analysis by clicking the <i title="report results" class="fa fa-edit"></i> icon on the bottom left of your screen or by pressing `ALT-enter` on your keyboard. 

If either a plot was created it can be customized using `ggplot2` commands (e.g., `plot(result) + labs(title = "Variable Importance")`). See <a href="https://radiant-rstats.github.io/docs/data/visualize.html" target="_blank">_Data > Visualize_</a> for details.

### R-functions

For an overview of related R-functions used by Radiant to estimate a naive Bayes classification model see <a href = "https://radiant-rstats.github.io/radiant.model/reference/index.html#section-model-naive-bayes" target="_blank">_Model > Naive Bayes_</a>

The key function from the `e1071` package used in the `nb` tool is `naiveBayes`.
